import time
from module.main import Normalizer

if __name__ == "__main__":


    # # Testing Date format
    # date_list = [
    #     "সেপ্টেম্বর ০৫ ২০২৩",
    #     "এপ্রিল ২০২৩" 
    #     "2023-04-05",  
    #     "06-04-2023", 
    #     "04/01/2023",  
    #     "07 April, 2023", 
    #     "Apr 1, 2023",  
    #     "2023/04/01", 
    #     "01-Apr-2023", 
    #     "01-Apr/2023",  
    #     "20230401",  
    #     "20042024",
    #     ["1", "4", "2025"]
    # ]
    # # # # number = "123456" or "২০২৩"
    # # number = "২০২৩"
    nrml = Normalizer()
    # # # print("++++++++++++++++++++ Date testing ++++++++++++++++++++++")
    # # print("Date format Testing : ", end ="", flush=True)
    # for date_ in date_list:
    #     start_time = time.time()
    #     formated_date = nrml.date_format(date_, language="bn")
    #     print(formated_date)
    # print("++++++++++++++++++++ end of Date testing ++++++++++++++++++++++")

    # print("++++++++++++++++++++ en number to bn number convert ++++++++++++++++++++++")
    # number = nrml.number_convert(number, language="bn")
    
    
    # number = nrml.number_convert(number, language="bn")

    # print("Bn Number : ", number)
    # print("++++++++++++++++++++ stop number convert ++++++++++++++++++++++")

    # # print("++++++++++++++++ Today +++++++++++++++++++++")
    # today_date = nrml.today(language="bn")
    # # today_date = nrml.today(language="en")
    # print(today_date)

    # # print("++++++++++++++++ End Today +++++++++++++++++++++")
    # # # print(today_date)
    # # print("++++++++++++++++ weekdays +++++++++++++++++++++")
    # weekdays = nrml.weekdays()
    # weekdays = nrml.weekdays(language="bn")
    # weekdays = nrml.weekdays(language="en")
    # weekdays = nrml.weekdays(day = "সোমবার")
    # weekdays = nrml.weekdays(day = "Monday")
    # print(weekdays)

    # # print("++++++++++++++++ end weekdays +++++++++++++++++++++")
    # # # print(weekdays)
    # # print("+++++++++++++++ seasons ++++++++++++++++++++++++")
    # seasons = nrml.seasons()
    # seasons = nrml.seasons(language="bn")
    # seasons = nrml.seasons(language="en")
    # seasons = nrml.seasons(seasons = "গ্রীষ্ম")
    # print(seasons)

    # # print("+++++++++++++++ end seasons ++++++++++++++++++++++++")

    # # print("+++++++++++++++++ months +++++++++++++++++++++++++++")
    # month = nrml.months()
    # month = nrml.months(month="মার্চ")
    # print(month)


    # text = "রাহিম ক্লাস ওয়ান এ ১ম, ১১তম ২২ তম ৩৩ তম, ১২৩৪ শতাব্দীতে ¥২০৩০.১২৩৪ বিবিধ  বাকেরগঞ্জ উপজেলার প্রায় 40 ভাগের পেশাই চাষাবাদ 80 and 40 ২২"
    
    # text = nrml.text_normalizer(text)

    # print(text)

    # input_texts = [
    #     "আমি এক দুই তিন চার পাঁচ টু থ্রি ফাইভ ছয় সেভেন এইট নাইন শূন্য আমার ফোন নাম্বার জিরো ওয়ান ডাবল সেভেন",
    #     "ওয়ান ডাবল নাইন টু",
    #     "একশ বিশ টাকা",
    #     "জিরো টু ডাবল ওয়ান",
    #     "জিরো ওয়ান ডাবল সেভেন থ্রি ডাবল ফাইভ নাইন থ্রি সেভেন নাইন",
    #     "আমার ফোন নম্বর জিরো ওয়ান ডাবল সেভেন থ্রি ডাবল ফাইভ নাইন থ্রি সেভেন নাইন",
    #     "ট্রিপল টু ওয়ান",
    #     "দুই হাজার চারশো বিশ",
    #     "দুই হাজার চারশ  বিশ",
    #     "হাজার বিশ",
    #     "ডাবল নাইন টু",
    #     "এক লক্ষ চার হাজার দুইশ",
    #     "এক লক্ষ চার হাজার দুইশ এক",
    #     "এক লক্ষ চার হাজার দুইশ এক টাকা এক দুই",
    #     "আমাকে এক লক্ষ দুই হাজার টাকা দেয়",
    #     "আমাকে এক লক্ষ দুই হাজার এক টাকা দেয় এন্ড তুমি বিশ হাজার টাকা নিও এন্ড এক লক্ষ চার হাজার দুইশ এক টাকা এক ডবল দুই",
    #     "ছয় হাজার বিশ",
    #     "আমার সাড়ে পাঁচ হাজার",
    #     "আমার সাড়ে তিনশ",
    #     "আড়াই হাজার",
    #     "আড়াই লক্ষ",
    #     "ডেরশ",
    #     "আমাকে ডেরশ টাকা দেয়",
    #     "সাড়ে পাঁচ কোটি টাকা",
    #     "সাড়ে 1254 টাকা",
    #     "জিরো",
    #     "একশ বিশ take একশ",
    #     "জিরো টু ডাবল ওয়ান",
    #     "জিরো টু ওয়ান ওয়ান",
    #     "থ্রি ফোর ফাইভ এইট",
    #     "একশ বিশ টাকা",
    #     "ডাবল ওয়ান ডবল টু",
    #     "জিরো ওয়ান টু",
    #     "থ্রি ফোর ফাইভ সিক্স",
    #     "সেভেন এইট নাইন টেন",
    #     "একশ দুইশ তিনশ",
    #     "চারশ পাঁচশ",
    #     "ছয়শ সাতশ",
    #     "আটশ নয়শ",
    #     "দশ তিরানব্বই",
    #     "ট্রিপল থ্রি টু",
    #     "শূন্য এক দুই তিন",
    #     "চার পাঁচ ছয় সাত",
    #     "আট নয় দশ এগারো",
    #     "বারো তেরো চৌদ্দ পনেরো",
    #     "ষোল সতেরো আঠারো উনিশ",
    #     "বিশ একুশ বাইশ তেইশ",
    #     "চব্বিশ পঁচিশ ছাব্বিশ সাতাশ",
    #     "আঠাশ ঊনত্রিশ ত্রিশ একত্রিশ",
    #     "বত্রিশ তেত্রিশ চৌত্রিশ পঁয়ত্রিশ",
    #     "ছত্রিশ সাঁইত্রিশ আটত্রিশ ঊনচল্লিশ",
    #     "চল্লিশ একচল্লিশ বিয়াল্লিশ তেতাল্লিশ",
    #     "চুয়াল্লিশ পঁয়তাল্লিশ ছেচল্লিশ সাতচল্লিশ",
    #     "আটচল্লিশ ঊনপঞ্চাশ পঞ্চাশ একান্ন",
    #     "বাহান্ন তিপ্পান্ন চুয়ান্ন পঞ্চান্ন",
    #     "ছাপ্পান্ন সাতান্ন আটান্ন ঊনষাট",
    #     "ষাট একষট্টি বাষট্টি তেষট্টি",
    #     "চৌষট্টি পঁয়ষট্টি ছেষট্টি সাতষট্টি",
    #     "আটষট্টি ঊনসত্তর সত্তর একাত্তর",
    #     "বাহাত্তর তিয়াত্তর চুয়াত্তর পঁচাত্তর",
    #     "ছিয়াত্তর সাতাত্তর আটাত্তর ঊনআশি",
    #     "আশি একাশি বিরাশি তিরাশি",
    #     "চুরাশি পঁচাশি ছিয়াশি সাতাশি",
    #     "আটাশি ঊননব্বই নব্বই একানব্বই",
    #     "বিরানব্বই তিরানব্বই চুরানব্বই পঁচানব্বই",
    #     "ছিয়ানব্বই সাতানব্বই আটানব্বই নিরানব্বই",
    #     "এক লক্ষ চার হাজার দুইশ এক টাকা এক দুই",
    #     "তিনশ পঁচিশ পাঁচশ",
    #     "তিনশ পঁচিশ পাঁচশ এক",
    #     "চা-পুন",
    #     "ওকে",
    #     "ডের আউটস্ট্যান্ডিং কত",
    #     "ডাবল",
    #     "নাইন ডাবল এইট",
    #     "দশ বারো এ এগুলা একশ একশ দুই",
    #     "এক লক্ষ তেত্রিশ চার"
    #     ]
    # for i in input_texts:
    #     print("="*40)
    #     print("input : ", i)
    #     text = nrml.word2number(i)
    #     print("output : ", text)
    #     print("="*40)
    # text = "সম্মেলনটি সেপ্টেম্বর ০৫ ২০২৩ তারিখে নির্ধারিত করা হয়েছে. এপ্রিল ২,০২৩"
    # formated_date = nrml.date_extraction(text)

    # print(formated_date)

    # text = "৫ বছরে নন-ক্যাডার পদে ৭,৪৪৭ জনকে নিয়োগের সুপারিশ করা হয়েছে 1,230"
    text = "৫ বছরে নন-ক্যাডার পদে ৭,৪৪৭ জনকে নিয়োগের সুপারিশ করা হয়েছে"

    # text = "শীঘ্রই ভিক্টোরিয়ার এক প্রতিনিধিদল আসছেন,শিলংয়ের ব্রুকসাইড হাউসে"

    # text = "The numbers are 10 নন-ক্যাডার 20.5  30, and 40.75. সুপারিশ ২০৩০.৩০ 12 23 45"
    # text = "This_is-a:test~sentence^with{some}unwanted[symbols]"

    print("input : ", text)
    text = nrml.text_normalizer(text)
    print("output : ", text)

